# we are kernel-xen-2.6!
%define xen_package_magic 1

Summary: The Linux kernel (the core of the Linux operating system)

# For a stable, released kernel, released_kernel should be 1. For rawhide
# and/or a kernel built from an rc or git snapshot, released_kernel should
# be 0.
%define released_kernel 1

# Versions of various parts

# Polite request for people who spin their own kernel rpms:
# please modify the "buildid" define in a way that identifies
# that the kernel isn't the stock distribution kernel, for example,
# by setting the define to ".local" or ".bz123456"
#
#% define buildid .local

# fedora_build defines which build revision of this kernel version we're
# building. Rather than incrementing forever, as with the prior versioning
# setup, the trick below will allow us to use the automatic CVS Revision keyword
# expansion, without breaking release number ordering.
#
# CVS Revision 1.42 was after RPM Release 2925.14. We do some math to
# increase the Release number starting from 2926
#
%define baserevision 42
%define baserelease 2926
%define fedora_build %(R="$Revision: 1.9 $"; \
	# be careful: '%%' becomes '%' below \
	R="${R%% \$}"; R="${R##: 1.}"; \
	# keep anything after the first dot, so this won't \
	# break horribily when using branches \
	# (e.g. 1.42.1.2.3.4 will become 1.2926.1.2.3.4) \
	r1="${R%%%%.*}";rest="${R#$r1}"; \
	# now, the math: \
	echo `expr '(' $r1 - %{baserevision} ')' + %{baserelease}`$rest)

# base_sublevel is the kernel version we're starting with and patching
# on top of -- for example, 2.6.22-rc7-git1 starts with a 2.6.21 base,
# which yields a base_sublevel of 21.
%define base_sublevel 21

## If this is a released kernel ##
%if 0%{?released_kernel}
# Do we have a 2.6.21.y update to apply?
%define stable_update 7
# Set rpm version accordingly
%if 0%{?stable_update}
%define stablerev .%{stable_update}
%endif
%define rpmversion 2.6.%{base_sublevel}%{?stablerev}

## The not-released-kernel case ##
%else
# The next upstream release sublevel (base_sublevel+1)
%define upstream_sublevel %(expr %{base_sublevel} + 1)
# The rc snapshot level
%define rcrev 0
# The git snapshot level
%define gitrev 17
# Set rpm version accordingly
%define rpmversion 2.6.%{upstream_sublevel}
%endif
# Nb: The above rcrev and gitrev values automagically define Patch00 and Patch01 below.

# What parts do we want to build?  We must build at least one kernel.
# These are the kernels that are built IF the architecture allows it.
# All should default to 1 (enabled) and be flipped to 0 (disabled)
# by later arch-specific checks.

# The following build options are enabled by default.
# Use either --without <opt> in your rpmbuild command or force values
# to 0 in here to disable them.
#
# standard kernel
%define with_up        %{?_without_up:        0} %{?!_without_up:        1}
# kernel-smp (only valid for ppc 32-bit, sparc64)
%define with_smp       %{?_without_smp:       0} %{?!_without_smp:       1}
# kernel-PAE (only valid for i686)
%define with_pae       %{?_without_pae:       0} %{?!_without_pae:       1}
# kernel-xen
%define with_xen       %{?_without_xen:       0} %{?!_without_xen:       1}
# kernel-kdump
%define with_kdump     %{?_without_kdump:     0} %{?!_without_kdump:     1}
# kernel-debug
%define with_debug     %{?_without_debug:     0} %{!?_without_debug:     1}
# kernel-doc
%define with_doc       %{?_without_doc:       0} %{?!_without_doc:       1}
# kernel-headers
%define with_headers   %{?_without_headers:   0} %{?!_without_headers:   1}
# kernel-debuginfo
%define with_debuginfo %{?_without_debuginfo: 0} %{!?_without_debuginfo: 1}

# Additional options for user-friendly one-off kernel building:
#
# Only build the base kernel (--with baseonly):
%define with_baseonly  %{?_with_baseonly:     1} %{?!_with_baseonly:     0}
# Only build the smp kernel (--with smponly):
%define with_smponly   %{?_with_smponly:      1} %{?!_with_smponly:      0}
# Only build the pae kernel (--with paeonly):
%define with_paeonly   %{?_with_paeonly:      1} %{?!_with_paeonly:      0}
# Only build the xen kernel (--with xenonly):
%define with_xenonly   %{?_with_xenonly:      1} %{?!_with_xenonly:      0}

# Whether or not to do C=1 builds with sparse
%define usesparse 0
%if "%fedora" >= "7"
%define usesparse 1
%endif

# defaults for the non-xen package:

# don't include xen patches
%define includexen 0

# don't build xen variant
%define with_xen 0

# Install vdso files
%define do_vdso_install 1


# Magic that will be enabled for the kernel-xen-2.6 Fedora package:
%if 0%{?xen_package_magic}

# Include the Xen patches
%define includexen 1

# Enable the xen variant
%define with_xen 1

# No vdso files
%define do_vdso_install 0

# Disable all other variants
%define with_xenonly 1

# No kernel-headers
%define with_headers 0


# We don't build the .config files dynamically, currently
%define preconfigured 1

# The main package is called kernel-xen-2.6
%define variant -xen-2.6

# However the -xen variant will be called kernel-xen
%define variantbase kernel

%endif   # (xen_package_magic)


# Set debugbuildsenabled to 1 for production (build separate debug kernels)
#  and 0 for rawhide (all kernels are debug kernels).
# See also 'make debug' and 'make release'.
%define debugbuildsenabled 0

# Want to build a vanilla kernel build without any non-upstream patches?
# (well, almost none, we need nonintconfig for build purposes). Default to 0 (off).
%define with_vanilla %{?_with_vanilla: 1} %{?!_with_vanilla: 0}

# pkg_release is what we'll fill in for the rpm Release: field
%if 0%{?released_kernel}
%define pkg_release %{fedora_build}%{?buildid}%{?dist}
%else
%if 0%{?rcrev}
%define rctag .rc%rcrev
%endif
%if 0%{?gitrev}
%define gittag .git%gitrev
%if !0%{?rcrev}
%define rctag .rc0
%endif
%endif
%define pkg_release 0.%{fedora_build}%{?rctag}%{?gittag}%{?buildid}%{?dist}
%endif

%define mainpkgname kernel%{?variant}
%if !0%{?variantbase:1}
%define variantbase %{mainpkgname}
%endif

# The kernel tarball/base version
%define kversion 2.6.%{base_sublevel}

%define make_target bzImage

%define xen_hv_version 3.2.0
%define xen_hv_dirname xen-%{xen_hv_version}
%define xen_flags verbose=y crash_debug=y
%define xen_target vmlinuz
%define xen_image vmlinuz

%define KVERREL %{PACKAGE_VERSION}-%{PACKAGE_RELEASE}
%define hdrarch %_target_cpu

%if 0%{!?nopatches:1}
%define nopatches 0
%endif

%if %{with_vanilla}
%define nopatches 1
%endif

%if %{nopatches}
%define includexen 0
%define with_xen 0
%define variant -vanilla
%else
%define variant_fedora -fedora
%endif

%define using_upstream_branch 0
%if 0%{?upstream_branch:1}
%define using_upstream_branch 1
%define variant -%{upstream_branch}%{?variant_fedora}
%define pkg_release %{upstream_branch_release}.%{pkg_release}
%endif

%if !%{debugbuildsenabled}
%define with_debug 0
%endif

%if !%{with_debuginfo}
%define _enable_debug_packages 0
%endif
%define debuginfodir /usr/lib/debug

# if requested, only build base kernel
%if %{with_baseonly}
%define with_smp 0
%define with_pae 0
%define with_xen 0
%define with_kdump 0
%define with_debug 0
%endif

# if requested, only build smp kernel
%if %{with_smponly}
%define with_up 0
%define with_pae 0
%define with_xen 0
%define with_kdump 0
%define with_debug 0
%endif

# if requested, only build pae kernel
%if %{with_paeonly}
%define with_up 0
%define with_smp 0
%define with_xen 0
%define with_kdump 0
%define with_debug 0
%endif

# if requested, only build xen kernel
%if %{with_xenonly}
%define with_up 0
%define with_smp 0
%define with_pae 0
%define with_kdump 0
%define with_debug 0
%endif

%define all_x86 i386 i586 i686

# These arches install vdso/ directories.
%define vdso_arches %{all_x86} x86_64 ppc ppc64

# Overrides for generic default options

# only ppc and sparc64 need separate smp kernels
%ifnarch ppc sparc64 alphaev56
%define with_smp 0
%endif

# pae is only valid on i686
%ifnarch i686
%define with_pae 0
%endif

# xen only builds on i686, x86_64 and ia64
%ifnarch i686 x86_64 ia64
%define with_xen 0
%endif

# only build kernel-kdump on ppc64
# (no relocatable kernel support upstream yet)
%ifnarch ppc64
%define with_kdump 0
%endif

# don't do debug builds on anything but i686 and x86_64
%ifnarch i686 x86_64
%define with_debug 0
%endif

# only package docs noarch
%ifnarch noarch
%define with_doc 0
%endif

# no need to build headers again for these arches,
# they can just use i386 and ppc64 and sparc headers
%ifarch i586 i686 ppc64iseries sparc64
%define with_headers 0
%endif

# don't build noarch kernels or headers (duh)
%ifarch noarch
%define with_up 0
%define with_headers 0
%define all_arch_configs kernel-%{version}-*.config
%endif

# sparse blows up on ppc64
%ifarch ppc64 ppc alpha sparc64
%define usesparse 0
%endif

# Per-arch tweaks

%ifarch %{all_x86}
%define all_arch_configs kernel-%{version}-i?86*.config
%define image_install_path boot
%define hdrarch i386
# we build always xen i686 HV with pae
%define xen_flags verbose=y crash_debug=y pae=y
%define kernel_image arch/x86/boot/bzImage
%endif

%ifarch x86_64
%define all_arch_configs kernel-%{version}-x86_64*.config
%define image_install_path boot
%define kernel_image arch/x86/boot/bzImage
%endif

%ifarch ppc64
%define all_arch_configs kernel-%{version}-ppc64*.config
%define image_install_path boot
%define make_target vmlinux
%define kernel_image vmlinux
%define kernel_image_elf 1
%define hdrarch powerpc
%endif

%ifarch s390x
%define all_arch_configs kernel-%{version}-s390x.config
%define image_install_path boot
%define make_target image
%define kernel_image arch/s390/boot/image
%define hdrarch s390
%endif

%ifarch sparc
# Yes, this is a hack. We want both sets of headers in the sparc.rpm
%define hdrarch sparc64
%endif

%ifarch sparc64
%define all_arch_configs kernel-%{version}-sparc64*.config
%define make_target image
%define kernel_image arch/sparc64/boot/image
%define image_install_path boot
%endif

%ifarch ppc
%define all_arch_configs kernel-%{version}-ppc{-,.}*config
%define image_install_path boot
%define make_target vmlinux
%define kernel_image vmlinux
%define kernel_image_elf 1
%define hdrarch powerpc
%endif

%ifarch ia64
%define all_arch_configs kernel-%{version}-ia64*.config
%define image_install_path boot/efi/EFI/redhat
%define make_target compressed
%define kernel_image vmlinux.gz
# ia64 xen HV doesn't building with debug=y at the moment
%define xen_flags verbose=y crash_debug=y
%define xen_target compressed
%define xen_image vmlinux.gz
%endif

%ifarch alpha alphaev56
%define all_arch_configs kernel-%{version}-alpha*.config
%define image_install_path boot
%define make_target vmlinux
%define kernel_image vmlinux
%endif

%if %{nopatches}
# XXX temporary until last vdso patches are upstream
%define vdso_arches %{nil}
%endif

%if %{nopatches}%{using_upstream_branch}
# Ignore unknown options in our config-* files.
# Some options go with patches we're not applying.
%define oldconfig_target loose_nonint_oldconfig
%else
%define oldconfig_target nonint_oldconfig
%endif

# To temporarily exclude an architecture from being built, add it to
# %nobuildarches. Do _NOT_ use the ExclusiveArch: line, because if we
# don't build kernel-headers then the new build system will no longer let
# us use the previous build of that package -- it'll just be completely AWOL.
# Which is a BadThing(tm).

# We don't build a kernel on i386; we only do kernel-headers there,
# and we no longer build for 31bit S390. Same for 32bit sparc.
%define nobuildarches i386 s390 sparc

%ifarch %nobuildarches
%define with_up 0
%define with_smp 0
%define with_pae 0
%define with_xen 0
%define with_kdump 0
%define with_debuginfo 0
%define _enable_debug_packages 0
%endif

%define with_pae_debug 0
%if %with_pae
%define with_pae_debug %{with_debug}
%endif

#
# Three sets of minimum package version requirements in the form of Conflicts:
# to versions below the minimum
#

#
# First the general kernel 2.6 required versions as per
# Documentation/Changes
#
%define kernel_dot_org_conflicts  ppp < 2.4.3-3, isdn4k-utils < 3.2-32, nfs-utils < 1.0.7-12, e2fsprogs < 1.37-4, util-linux < 2.12, jfsutils < 1.1.7-2, reiserfs-utils < 3.6.19-2, xfsprogs < 2.6.13-4, procps < 3.2.5-6.3, oprofile < 0.9.1-2

#
# Then a series of requirements that are distribution specific, either
# because we add patches for something, or the older versions have
# problems with the newer kernel or lack certain things that make
# integration in the distro harder than needed.
#
%define package_conflicts initscripts < 7.23, udev < 063-6, iptables < 1.3.2-1, ipw2200-firmware < 2.4, selinux-policy-targeted < 1.25.3-14

#
# The ld.so.conf.d file we install uses syntax older ldconfig's don't grok.
#
%define kernel_xen_conflicts glibc < 2.3.5-1, xen < 3.0.1

# xen-hypervisor-abi provides (bug #269581)
%define kernel_xen_provides xen-hypervisor-abi = 3.2

# upto and including kernel 2.4.9 rpms, the 4Gb+ kernel was called kernel-enterprise
# now that the smp kernel offers this capability, obsolete the old kernel
%define kernel_smp_obsoletes kernel-enterprise < 2.4.10
%define kernel_PAE_obsoletes kernel-smp < 2.6.17

#
# Packages that need to be installed before the kernel is, because the %post
# scripts use them.
#
%define kernel_prereq  fileutils, module-init-tools, initscripts >= 8.11.1-1, mkinitrd >= 6.0.9-7

#
# This macro does requires, provides, conflicts, obsoletes for a kernel package.
#	%%kernel_reqprovconf <subpackage>
# It uses any kernel_<subpackage>_conflicts and kernel_<subpackage>_obsoletes
# macros defined above.
#
%define kernel_reqprovconf \
Provides: kernel = %{rpmversion}-%{pkg_release}\
Provides: kernel-%{_target_cpu} = %{rpmversion}-%{pkg_release}%{?1}\
Provides: kernel-drm = 4.3.0\
Provides: kernel-drm-nouveau = 10\
Requires(pre): %{kernel_prereq}\
Conflicts: %{kernel_dot_org_conflicts}\
Conflicts: %{package_conflicts}\
%{?1:%{expand:%%{?kernel_%{1}_conflicts:Conflicts: %%{kernel_%{1}_conflicts}}}}\
%{?1:%{expand:%%{?kernel_%{1}_obsoletes:Obsoletes: %%{kernel_%{1}_obsoletes}}}}\
%{?1:%{expand:%%{?kernel_%{1}_provides:Provides: %%{kernel_%{1}_provides}}}}\
# We can't let RPM do the dependencies automatic because it'll then pick up\
# a correct but undesirable perl dependency from the module headers which\
# isn't required for the kernel proper to function\
AutoReq: no\
AutoProv: yes\
%{nil}

Name: %{mainpkgname}
Group: System Environment/Kernel
License: GPLv2
URL: http://www.kernel.org/
Version: %{rpmversion}
Release: %{pkg_release}
# non-xen kernel says:
# > DO NOT CHANGE THE 'ExclusiveArch' LINE TO TEMPORARILY EXCLUDE AN ARCHITECTURE BUILD.
# > SET %%nobuildarches (ABOVE) INSTEAD
# However kernel-xen-2.6 can ignore this warning because it is not responsible
# for generating the kernel-headers packages.
ExclusiveArch: noarch i686 x86_64
ExclusiveOS: Linux

%kernel_reqprovconf
%ifarch x86_64
Obsoletes: kernel-smp
%endif


#
# List the packages used during the kernel build
#
BuildRequires: module-init-tools, patch >= 2.5.4, bash >= 2.03, sh-utils, tar
BuildRequires: bzip2, findutils, gzip, m4, perl, make >= 3.78, diffutils, gawk
BuildRequires: gcc >= 3.4.2, binutils >= 2.12, redhat-rpm-config
%if %{usesparse}
BuildRequires: sparse >= 0.3
%endif
BuildConflicts: rhbuildsys(DiskFree) < 500Mb

# needed for xen mkheader.py
BuildRequires: python

%define fancy_debuginfo 0
%if %{with_debuginfo}
%if "%fedora" > "7"
%define fancy_debuginfo 1
%endif
%endif

%if %{fancy_debuginfo}
# Fancy new debuginfo generation introduced in Fedora 8.
BuildRequires: rpm-build >= 4.4.2.1-4
%define debuginfo_args --strict-build-id
%endif

Source0: ftp://ftp.kernel.org/pub/linux/kernel/v2.6/linux-%{kversion}.tar.bz2
Source1: xen-%{xen_hv_version}.tar.gz
Source2: Config.mk

Source10: COPYING.modules
Source11: genkey
Source14: find-provides
Source15: merge.pl

Source20: Makefile.config
Source21: config-debug
Source22: config-nodebug
Source23: config-generic
Source24: config-xen-generic
Source25: config-rhel-generic
Source26: config-rhel-x86-generic

Source30: config-x86-generic
Source31: config-i586
Source32: config-i686
Source33: config-i686-PAE
Source34: config-xen-x86

Source40: config-x86_64-generic
Source41: config-xen-x86_64

Source50: config-powerpc-generic
Source51: config-powerpc32-generic
Source52: config-powerpc32-smp
Source53: config-powerpc64
Source54: config-powerpc64-kdump

Source60: config-ia64-generic
Source61: config-ia64
Source62: config-xen-ia64

Source70: config-s390x

Source90: config-sparc64-generic
Source91: config-sparc64
Source92: config-sparc64-smp

Source100: kernel-%{version}-i686-xen.config
Source101: kernel-%{version}-x86_64-xen.config

%if %{using_upstream_branch}
### BRANCH PATCH ###
%else
# Here should be only the patches up to the upstream canonical Linus tree.

# For a stable release kernel
%if 0%{?stable_update}
Patch00: patch-2.6.%{base_sublevel}.%{stable_update}.bz2

# non-released_kernel case
# These are automagically defined by the rcrev and gitrev values set up
# near the top of this spec file.
%else
%if 0%{?rcrev}
Patch00: patch-2.6.%{upstream_sublevel}-rc%{rcrev}.bz2
%if 0%{?gitrev}
Patch01: patch-2.6.%{upstream_sublevel}-rc%{rcrev}-git%{gitrev}.bz2
%endif
%else
# pre-{base_sublevel+1}-rc1 case
%if 0%{?gitrev}
Patch00: patch-2.6.%{base_sublevel}-git%{gitrev}.bz2
%endif
%endif
%endif

%endif

%if !%{nopatches}

# Patches 10 through 99 are for things that are going upstream really soon.
Patch11: nouveau-drm.patch

Patch12: linux-2.6-fix-pmops-1.patch
Patch13: linux-2.6-fix-pmops-2.patch
Patch14: linux-2.6-fix-pmops-3.patch
Patch15: linux-2.6-fix-pmops-4.patch

# DRM bits for 965
Patch17: linux-2.6-i965gm-support.patch

# enable sysrq-c on all kernels, not only kexec
Patch20: linux-2.6-sysrq-c.patch

# utrace
Patch30: linux-2.6-utrace-sig_kernel-macros.patch
Patch31: linux-2.6-utrace-recalc_sigpending_and_wake.patch
Patch32: linux-2.6-utrace-tracehook.patch
Patch33: linux-2.6-utrace-tracehook-ia64.patch
Patch34: linux-2.6-utrace-tracehook-sparc64.patch
Patch35: linux-2.6-utrace-tracehook-s390.patch
Patch36: linux-2.6-utrace-tracehook-um.patch
Patch37: linux-2.6-utrace-regset.patch
Patch38: linux-2.6-utrace-regset-ia64.patch
Patch39: linux-2.6-utrace-regset-sparc64.patch
Patch40: linux-2.6-utrace-regset-s390.patch
Patch41: linux-2.6-utrace-core.patch
Patch42: linux-2.6-utrace-ptrace-compat.patch
Patch43: linux-2.6-utrace-ptrace-compat-ia64.patch
Patch44: linux-2.6-utrace-ptrace-compat-sparc64.patch
Patch45: linux-2.6-utrace-ptrace-compat-s390.patch

#xen bits of utrace patches:
Patch52: linux-2.6-utrace-tracehook-xen.patch
Patch62: linux-2.6-utrace-ptrace-compat-xen.patch


# build-id stuff (http://fedoraproject.org/wiki/Releases/FeatureBuildId)
Patch70: linux-2.6-use-build-id-ld-option.patch

# Patches 100 through 500 are meant for architecture patches

# 200 - 299   x86(-64)

Patch200: linux-2.6-x86-tune-generic.patch
Patch201: linux-2.6-x86-vga-vidfail.patch
Patch202: linux-2.6-x86-64-edac-support.patch
Patch203: linux-2.6-x86_64-silence-up-apic-errors.patch
Patch204: linux-2.6-x86_64-silence-up-apic-errors-xen.patch
Patch205: linux-2.6-x86-dont-delete-cpu_devs-data.patch
Patch206: linux-2.6-x86-fsc-interrupt-controller-quirk.patch
Patch207: linux-2.6-x86-dell-hpet.patch
Patch209: linux-2.6-x86-64_pmtrace.patch


# 300 - 399   ppc(64)
Patch300: linux-2.6-g5-therm-shutdown.patch
Patch301: linux-2.6-powerpc-slabalign.patch
Patch302: linux-2.6-ppc-data-exception.patch
Patch303: linux-2.6-ppc32-ucmpdi2.patch
Patch304: linux-2.6-ibmvscsi-schizo.patch
Patch305: linux-2.6-pmac-zilog.patch
Patch306: linux-2.6-powerpc-reserve-initrd-1.patch
Patch307: linux-2.6-powerpc-reserve-initrd-2.patch
Patch308: linux-2.6-cell-spu-device-tree.patch
Patch309: linux-2.6-cell-spufs-fixes.patch

Patch310: linux-2.6-common-uevent.patch
Patch311: linux-2.6-uevent-macio.patch
Patch312: linux-2.6-uevent-of_platform.patch
Patch313: linux-2.6-uevent-ebus.patch

Patch330: linux-2.6-powermac-generic-suspend-1.patch
Patch331: linux-2.6-powermac-generic-suspend-2.patch
Patch332: linux-2.6-powermac-generic-suspend-3.patch
Patch333: linux-2.6-powermac-generic-suspend-4.patch

Patch340: linux-2.6-mpc52xx-sdma.patch
Patch341: linux-2.6-mpc52xx-fec.patch

# Patches from ps3-linux-patches.git (2007-05-04)
Patch350: linux-2.6-ps3-stable-patches.patch
Patch351: linux-2.6-ps3-smp-boot.patch
Patch352: linux-2.6-ps3-system-bus-rework.patch
Patch353: linux-2.6-ps3-kexec.patch
Patch354: linux-2.6-ps3-gelic.patch
Patch355: linux-2.6-ps3-gelic-wireless.patch
Patch356: linux-2.6-ps3-ehci-iso.patch
Patch357: linux-2.6-ps3-clear-spu-irq.patch
Patch358: linux-2.6-ps3-wrap-spu-runctl.patch
# Ignore the SPE logo bits. Cute, but not exactly necessary
Patch359: linux-2.6-ps3-storage.patch
Patch360: linux-2.6-ps3-sound.patch
Patch361: linux-2.6-ps3-device-init.patch
Patch362: linux-2.6-ps3-system-bus-rework-2.patch

# And then some minor tweaks...
Patch370: linux-2.6-ps3-memory-probe.patch
Patch371: linux-2.6-ps3-legacy-ioport.patch
Patch372: linux-2.6-ps3fb-panic.patch
Patch373: linux-2.6-ps3-ethernet-modular.patch
Patch374: linux-2.6-ps3-sound-autoload.patch
Patch375: linux-2.6-ps3-ethernet-autoload.patch
Patch376: linux-2.6-ps3av-export-header.patch
Patch377: linux-2.6-ps3-usb-autoload.patch



# 400 - 499   ia64

# 500 - 599   s390(x)

# 600 - 699   sparc(64)

#
# Patches 800 through 899 are reserved for bugfixes to the core system
# and patches related to how RPMs are build
#
Patch800: linux-2.6-build-nonintconfig.patch

# Exec-shield.
Patch810: linux-2.6-execshield.patch
Patch811: linux-2.6-execshield-xen.patch

# Module signing infrastructure.
Patch900: linux-2.6-modsign-mpilib.patch
Patch901: linux-2.6-modsign-crypto.patch
Patch902: linux-2.6-modsign-include.patch
Patch903: linux-2.6-modsign-verify.patch
Patch904: linux-2.6-modsign-ksign.patch
Patch905: linux-2.6-modsign-core.patch
Patch906: linux-2.6-modsign-script.patch

# 950 - 999 Xen
# linux-2.6.21.7-xen-3.1.0.patch patch generated by
# 'hg diff -rd91e11100fb0 -rd610071e6045 -X .hgtags' on
# http://hg.et.redhat.com/kernel-dev/ehabkost/linux-2.6.21.y-xen-3.1.0
Patch950: linux-2.6.21.7-xen-3.1.0.patch.bz2
Patch960: linux-2.6-xen-blkfront-wait-add.patch
Patch961: linux-2.6-xen-backwards-time.patch
Patch962: linux-2.6-xen-irq_vector-uninitialize.patch

# Fix sleazy-FPU implementation
Patch963: linux-2.6-xen-sleazy-fpu-i386.patch
Patch964: linux-2.6-xen-sleazy-fpu-x86_64.patch

# Fix VM_PFNMAP BUG_ON()
Patch965: linux-2.6-xen-privcmd-use-nopfn.patch

# Remove bogus WARN_ON() from synchronize_irq() (bug #293451)
Patch966: linux-2.6-xen-fix-irq-warn-mismerge.patch

# virt-install --nographics --paravirt fix (bug #348931)
Patch967: linux-2.6-xen-paravirt-nographics-pvfb-fix.patch

#
# Patches 1000 to 5000 are reserved for bugfixes to drivers and filesystems
#

Patch1010: linux-2.6-debug-sizeof-structs.patch
Patch1011: linux-2.6-debug-slab-backtrace.patch
Patch1012: linux-2.6-debug-nmi-timeout.patch
Patch1013: linux-2.6-debug-taint-vm.patch
Patch1015: linux-2.6-debug-spinlock-taint.patch
Patch1016: linux-2.6-debug-extra-warnings.patch
Patch1019: linux-2.6-debug-must_check.patch
Patch1020: linux-2.6-debug-no-quiet.patch
Patch1021: linux-2.6-debug-boot-delay.patch
Patch1022: linux-2.6-debug-sysfs-crash-debugging.patch
Patch1023: linux-2.6-debug-sysfs-crash-debugging-xen.patch

# Restrict /dev/mem usage.
Patch1050: linux-2.6-devmem.patch
Patch1051: linux-2.6-devmem-xen.patch

# Provide read only /dev/crash driver.
Patch1060: linux-2.6-crash-driver.patch
Patch1061: linux-2.6-crash-driver-xen.patch

# SCSI bits.
Patch1100: linux-2.6-scsi-bounce-isa.patch
Patch1106: linux-2.6-scsi-cpqarray-set-master.patch
Patch1107: linux-2.6-3w-9xxx-mem_len.patch

# NIC driver fixes
Patch1300: linux-2.6-net-e1000-no-msi-warning.patch

# Filesystem stuff.
# Squashfs
Patch1400: linux-2.6-squashfs.patch
# GFS2
Patch1410: linux-2.6-gfs2-update.patch


# Networking core.
Patch1500: linux-2.6-net-silence-noisy-printks.patch

# Misc bits.
Patch1600: linux-2.6-module_version.patch
Patch1601: linux-2.6-sha_alignment.patch
Patch1610: linux-2.6-input-kill-stupid-messages.patch
Patch1620: linux-2.6-ondemand-timer.patch
Patch1630: linux-2.6-kvm-19.patch
Patch1631: linux-2.6-amd-disabled-svm-detect.patch
Patch1632: linux-2.6-amd-disabled-svm-detect-msr-1.patch
Patch1633: linux-2.6-kvm-reinit-real-mode-tss.patch
Patch1650: linux-2.6-serial-460800.patch
Patch1660: linux-2.6-mm-udf-fixes.patch
Patch1661: linux-2.6-udf-2.6.22-rc2-1-udf_data_corruption.patch
Patch1662: linux-2.6-udf-2.6.22-rc4-1-udf_block_leak.patch
Patch1670: linux-2.6-sysfs-inode-allocator-oops.patch
# checksum fixes (bug #223258)
Patch1665: linux-2.6-disable-netback-checksum.patch

Patch1690: linux-2.6-PT_LOAD-align.patch
Patch1700: linux-2.6-dvb-spinlock.patch
Patch1710: linux-2.6-nfs-noreaddirplus.patch
Patch1711: linux-2.6-nfs-missing-braces.patch
Patch1720: linux-2.6-proc-self-maps-fix.patch
Patch1730: linux-2.6-suspend-ordering.patch
Patch1740: linux-2.6-softlockup-disable.patch
Patch1770: linux-2.6-optimise-spinlock-debug.patch
Patch1771: linux-2.6-silence-noise.patch
Patch1781: linux-2.6-softirq-printout-irq-trace-events.patch
Patch1791: linux-2.6-libertas.diff
Patch1792: linux-2.6-olpc-touchpad.diff
Patch1793: linux-2.6-raid-autorun.patch
Patch1794: linux-2.6-i82875-edac-pci-setup.patch
Patch1795: linux-2.6-crap-sysfs-workaround.patch
Patch1796: linux-2.6-oprofile-0.9.3.patch

# SELinux/audit patches.
Patch1801: linux-2.6-selinux-mprotect-checks.patch

# no external module should use these symbols.
Patch1910: linux-2.6-unexport-symbols.patch

# VM bits.
Patch2000: linux-2.6-vm-invalidate_mapping_pages-cond-resched.patch
Patch2001: linux-2.6-vm-silence-atomic-alloc-failures.patch

# Tweak some defaults.
Patch2100: linux-2.6-defaults-fat-utf8.patch
Patch2103: linux-2.6-defaults-unicode-vt.patch
Patch2105: linux-2.6-defaults-nonmi.patch
Patch2106: linux-2.6-defaults-pci_no_msi_mmconf.patch
Patch2107: linux-2.6-usb-autosuspend-default-disable.patch

# SATA Bits
Patch2201: linux-2.6-libata-hpa.patch
Patch2202: linux-2.6-libata-sata_nv-adma.patch
Patch2203: linux-2.6-libata-ali-atapi-dma.patch
Patch2204: linux-2.6-ata-quirk.patch
Patch2206: linux-2.6-libata-sata_nv-wildcard-removal.patch
Patch2207: linux-2.6-libata-pata-pcmcia-new-ident.patch
Patch2208: linux-2.6-libata-atiixp-ids.patch
Patch2209: linux-2.6-libata-pata-hpt3x2n-correct-revision-boundary.patch
Patch2210: linux-2.6-libata-pata-sis-fix-timing.patch
Patch2211: linux-2.6-libata-setxfer.patch
Patch2212: linux-2.6-libata_ali_max_dma_speed.patch
Patch2213: linux-2.6-ata-use-pio-for-non-16-byte-xfers.patch
Patch2214: linux-2.6-ata-call-check-dma-with-qc-prepared.patch
Patch2215: linux-2.6-libata-ich8m-add-pciid.patch
Patch2216: linux-2.6-libata-pata_dma-param.patch
Patch2217: linux-2.6-libata-ncq-blacklist-2.6.22-rc7.patch
Patch2218: linux-2.6-libata-pata_it821x-partly-fix-dma.patch

# ATA spindown
Patch2220: linux-2.6-2110_scsi-sd-printing.patch
Patch2221: linux-2.6-2111_sd-start-stop.patch
Patch2222: linux-2.6-2112_libata-suspend.patch
Patch2223: linux-2.6-2113_libata-spindown-compat.patch
Patch2224: linux-2.6-2114_libata-shutdown-warning.patch
Patch2225: linux-2.6-2115_libata-spindown-status.patch
Patch2226: linux-2.6-2116_libata-remove-spindown-compat.patch
Patch2227: linux-2.6-2117_sata-via-suspend.patch
Patch2228: linux-2.6-2118_scsi-constants.patch

# Wireless bits
Patch2300: linux-2.6-wireless.patch
Patch2301: git-wireless-dev.patch
Patch2302: linux-2.6-bcm43xx-pci-neuter.patch

# Assorted dyntick/clock/timer fixes.
Patch2402: linux-2.6-acpi-keep-tsc-stable-when-lapic-timer-c2-ok-is-set.patch
Patch2403: linux-2.6-clockevents-fix-resume-logic.patch

# ACPI bits
Patch2500: linux-2.6-acpi-unblacklist-dell-gx240.patch
Patch2501: linux-2.6-acpi-dock-oops.patch
Patch2502: linux-2.6-acpi-boot-regression.patch
Patch2503: linux-2.6-acpi-preserve-ebx-in-acpi_copy_wakeup_routine.patch

# Excessive wakeups.
Patch2600: linux-2.6-wakeups-hdaps.patch

Patch3000: linux-2.6-acpi-config_pm-poweroff.patch

# Add the new firewire stack. Diff between the v2.6.20 tag and commit
# 5468ee91741092a711327628c7c524a4d4d0cd6a in the linux1394 git tree.
Patch5000: linux-2.6-firewire.patch
Patch5001: linux-2.6-firewire-be32-fix.patch

#
# 10000 to 20000 is for stuff that has to come last due to the
# amount of drivers they touch. But only these should go here.
# Not patches you're too lazy for to put in the proper place.
#

# Xen hypervisor patches (20000+)
Patch20000: xen-compile-fix.patch
Patch20001: xen-version-strings.patch

# Xen ld --build-id option
# (http://fedoraproject.org/wiki/Releases/FeatureBuildId)
Patch20004: xen-build-id.patch

# Include build-id notes on xen-syms
Patch20005: xen-syms-build-id.patch




%endif

BuildRoot: %{_tmppath}/%{mainpkgname}-%{KVERREL}-root-%{_target_cpu}

%description
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system: memory allocation, process allocation, device
input and output, etc.


%package -n %{mainpkgname}-doc
Summary: Various documentation bits found in the kernel source
Group: Documentation
%description -n %{mainpkgname}-doc
This package contains documentation files from the kernel
source. Various bits of information about the Linux kernel and the
device drivers shipped with it are documented in these files.

You'll want to install this package if you need a reference to the
options that can be passed to Linux kernel modules at load time.


%package -n %{mainpkgname}-headers
Summary: Header files for the Linux kernel for use by glibc
Group: Development/System
Obsoletes: glibc-kernheaders
Provides: glibc-kernheaders = 3.0-46
%description -n %{mainpkgname}-headers
Kernel-headers includes the C header files that specify the interface
between the Linux kernel and userspace libraries and programs.  The
header files define structures and constants that are needed for
building most standard programs and are also needed for rebuilding the
glibc package.


%package -n %{mainpkgname}-debuginfo-common
Summary: Kernel source files used by %{mainpkgname}-debuginfo packages
Group: Development/Debug
Provides: %{mainpkgname}-debuginfo-common-%{_target_cpu} = %{KVERREL}
%description -n %{mainpkgname}-debuginfo-common
This package is required by %{mainpkgname}-debuginfo subpackages.
It provides the kernel source files common to all builds.


#
# This macro creates a kernel-<subpackage>-debuginfo package.
#	%%kernel_debuginfo_package <subpackage>
#
%define kernel_debuginfo_package() \
%package -n %{variantbase}-%{?1:%{1}-}debuginfo\
Summary: Debug information for package %{variantbase}%{?1:-%{1}}\
Group: Development/Debug\
Requires: %{mainpkgname}-debuginfo-common-%{_target_cpu} = %{KVERREL}\
Provides: %{variantbase}%{?1:-%{1}}-debuginfo-%{_target_cpu} = %{KVERREL}\
AutoReqProv: no\
%description -n %{variantbase}%{?1:-%{1}}-debuginfo\
This package provides debug information for package %{variantbase}%{?1:-%{1}}.\
This is required to use SystemTap with %{variantbase}%{?1:-%{1}}-%{KVERREL}.\
%{expand:%%global debuginfo_args %{?debuginfo_args} -p '/.*/%%{KVERREL}%{?1:-?%{1}}(-%%{_target_cpu})?/.*|/.*%%{KVERREL}%{?1}' -o debuginfo%{?1}.list}\
%{nil}

#
# This macro creates a kernel-<subpackage>-devel package.
#	%%kernel_devel_package <subpackage> <pretty-name>
#
%define kernel_devel_package() \
%package -n %{variantbase}-%{?1:%{1}-}devel\
Summary: Development package for building kernel modules to match the %{?2:%{2} }kernel\
Group: System Environment/Kernel\
Provides: kernel%{?1:-%{1}}-devel-%{_target_cpu} = %{rpmversion}-%{release}\
Provides: kernel-devel-%{_target_cpu} = %{rpmversion}-%{release}%{?1}\
Provides: kernel-devel = %{rpmversion}-%{release}%{?1}\
AutoReqProv: no\
Requires(pre): /usr/bin/find\
%description -n %{variantbase}%{?1:-%{1}}-devel\
This package provides kernel headers and makefiles sufficient to build modules\
against the %{?2:%{2} }kernel package.\
%{nil}

#
# This macro creates a kernel-<subpackage> and its -devel and -debuginfo too.
#	%%define variant_summary The Linux kernel compiled for <configuration>
#	%%kernel_variant_package [-n <pretty-name>] <subpackage>
#
%define kernel_variant_package(n:) \
%package -n %{variantbase}-%1\
Summary: %{variant_summary}\
Group: System Environment/Kernel\
%kernel_reqprovconf\
%{expand:%%kernel_devel_package %1 %{!?-n:%1}%{?-n:%{-n*}}}\
%{expand:%%kernel_debuginfo_package %1}\
%{nil}


# First the auxiliary packages of the main kernel package.
%kernel_devel_package
%kernel_debuginfo_package


# Now, each variant package.

%define variant_summary The Linux kernel compiled for SMP machines
%kernel_variant_package -n SMP smp
%description -n %{variantbase}-smp
This package includes a SMP version of the Linux kernel. It is
required only on machines with two or more CPUs as well as machines with
hyperthreading technology.

Install the kernel-smp package if your machine uses two or more CPUs.


%define variant_summary The Linux kernel compiled for PAE capable machines
%kernel_variant_package PAE
%description -n %{variantbase}-PAE
This package includes a version of the Linux kernel with support for up to
64GB of high memory. It requires a CPU with Physical Address Extensions (PAE).
The non-PAE kernel can only address up to 4GB of memory.
Install the kernel-PAE package if your machine has more than 4GB of memory.


%define variant_summary The Linux kernel compiled with extra debugging enabled for PAE capable machines
%kernel_variant_package PAE-debug
%description -n %{variantbase}-PAE-debug
This package includes a version of the Linux kernel with support for up to
64GB of high memory. It requires a CPU with Physical Address Extensions (PAE).
The non-PAE kernel can only address up to 4GB of memory.
Install the kernel-PAE package if your machine has more than 4GB of memory.

This variant of the kernel has numerous debugging options enabled.
It should only be installed when trying to gather additional information
on kernel bugs, as some of these options impact performance noticably.


%define variant_summary The Linux kernel compiled with extra debugging enabled
%kernel_variant_package debug
%description -n %{variantbase}-debug
The kernel package contains the Linux kernel (vmlinuz), the core of any
Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.

This variant of the kernel has numerous debugging options enabled.
It should only be installed when trying to gather additional information
on kernel bugs, as some of these options impact performance noticably.


%define variant_summary The Linux kernel compiled for Xen VM operations
%kernel_variant_package -n Xen xen
%description -n %{variantbase}-xen
This package includes version of the Linux kernel which
runs in a Xen VM. It works for both privileged and unprivileged guests.


%define variant_summary A minimal Linux kernel compiled for crash dumps
%kernel_variant_package kdump
%description -n %{variantbase}-kdump
This package includes a kdump version of the Linux kernel. It is
required only on machines which will use the kexec-based kernel crash dump
mechanism.


%prep
# do a few sanity-checks for --with *only builds
%if %{with_baseonly}
%if !%{with_up}
echo "Cannot build --with baseonly, up build is disabled"
exit 1
%endif
%endif

%if %{with_smponly}
%if !%{with_smp}
echo "Cannot build --with smponly, smp build is disabled"
exit 1
%endif
%endif

%if %{with_paeonly}
%if !%{with_pae}
echo "Cannot build --with paeonly, pae build is disabled"
exit 1
%endif
%endif

# If the package has a variantbase set, the up (no variant suffix) packages would
# be inconsistent (sometimes using %%{mainpkgname}, other using %%{variantbase})
#
# As we only use variantbase on the kernel-xen package (that has with_up
# disabled), it is better to not support it than making the spec file
# more complex by adding magic to allow generating separated %%package sessions
# if necessary (the up variant has no %%package session because it has
# no subpackage suffix, but it uses the %%post and %%files section generation
# macros).
%if "%{mainpkgname}"!="%{variantbase}" && %{with_up}
echo "Setting %%{variantbase}!=%%{mainpkgname} is not support when building up kernels"
exit 1
%endif




# First we unpack the kernel tarball.
# If this isn't the first make prep, we use links to the existing clean tarball
# which speeds things up quite a bit.
if [ ! -d kernel-%{kversion}/vanilla ]; then
  # Ok, first time we do a make prep.
  rm -f pax_global_header
%setup -q -n kernel-%{kversion} -c
  mv linux-%{kversion} vanilla
else
  # We already have a vanilla dir.
  cd kernel-%{kversion}
  if [ -d linux-%{kversion}.%{_target_cpu} ]; then
     # Just in case we ctrl-c'd a prep already
     rm -rf deleteme.%{_target_cpu}
     # Move away the stale away, and delete in background.
     mv linux-%{kversion}.%{_target_cpu} deleteme.%{_target_cpu}
     rm -rf deleteme.%{_target_cpu} &
  fi
fi

cp -rl vanilla linux-%{kversion}.%{_target_cpu}

cd linux-%{kversion}.%{_target_cpu}

%if 0%{?preconfigured}
  # The .config files were already generated
  cp $RPM_SOURCE_DIR/kernel-%{kversion}*.config .
%else
  # Drop some necessary files from the source dir into the buildroot
  cp $RPM_SOURCE_DIR/config-* .
  cp %{SOURCE15} .

  # Dynamically generate kernel .config files from config-* files
  make -f %{SOURCE20} VERSION=%{version} configs
%endif


#if a rhel kernel, apply the rhel config options
%if 0%{?rhel}
  for i in %{all_arch_configs}
  do
    mv $i $i.tmp
    ./merge.pl config-rhel-generic $i.tmp > $i
    rm $i.tmp
  done
  for i in kernel-%{version}-{i586,i686,i686-PAE,x86_64}*.config
  do
    echo i is this file  $i
    mv $i $i.tmp
    ./merge.pl config-rhel-x86-generic $i.tmp > $i
    rm $i.tmp
  done
%endif

patch_command='patch -p1 -F1 -s'
ApplyPatch()
{
  local patch=$1
  shift
  if [ ! -f $RPM_SOURCE_DIR/$patch ]; then
    exit 1;
  fi
  case "$patch" in
  *.bz2) bunzip2 < "$RPM_SOURCE_DIR/$patch" | $patch_command ${1+"$@"} ;;
  *.gz) gunzip < "$RPM_SOURCE_DIR/$patch" | $patch_command ${1+"$@"} ;;
  *) $patch_command ${1+"$@"} < "$RPM_SOURCE_DIR/$patch" ;;
  esac
}

%if %{using_upstream_branch}
### BRANCH APPLY ###
%else

# Update to latest upstream.
# released_kernel with stable_update available case
%if 0%{?stable_update}
ApplyPatch patch-2.6.%{base_sublevel}.%{stable_update}.bz2

# non-released_kernel case
%else
%if 0%{?rcrev}
ApplyPatch patch-2.6.%{upstream_sublevel}-rc%{rcrev}.bz2
%if 0%{?gitrev}
ApplyPatch patch-2.6.%{upstream_sublevel}-rc%{rcrev}-git%{gitrev}.bz2
%endif
%else
# pre-{base_sublevel+1}-rc1 case
%if 0%{?gitrev}
ApplyPatch patch-2.6.%{base_sublevel}-git%{gitrev}.bz2
%endif
%endif
%endif

%endif


%if !%{nopatches}

#
# Xen
#
%if %{includexen}
#
# Apply the main xen patch...
#%patch951 -p1
%patch950 -p1 -b .p.xen
#
# ... and back out all the tpm additions, they need fixing
#
for f in `find drivers/char/tpm -type f -name "*.p.xen"` ; do \
    g=`dirname $f`/`basename $f .p.xen`; \
    mv "$f" "$g"; \
    if [ ! -s "$g" ] ; then rm -f "$g" ; fi; \
done
# Delete the rest of the backup files, they just confuse the build later
find -name "*.p.xen" | xargs rm -f

# Xen utrace
%patch960 -p1
%patch961 -p1
%patch962 -p1
%patch963 -p1
%patch964 -p1
%patch965 -p1
%patch966 -p1
%patch967 -p1
%endif


# Patches 10 through 100 are meant for core subsystem upgrades
%patch11 -p1

# Power management fixes
%patch12 -p1
%patch13 -p1
%patch14 -p1
%patch15 -p1

# DRM support for 965GM
%patch17 -p1

# sysrq works always
%patch20 -p1

# Roland's utrace ptrace replacement.
%patch30 -p1
%patch31 -p1
%patch32 -p1
%patch33 -p1
%patch34 -p1
%patch35 -p1
%patch36 -p1
%patch37 -p1
%patch38 -p1
%patch39 -p1
%patch40 -p1
%patch41 -p1
%patch42 -p1
%patch43 -p1
%patch44 -p1
%patch45 -p1

%if %{includexen}
#xen bits of utrace
%patch52 -p1
%patch62 -p1
%endif

# Use --build-id option
%patch70 -p1

# Architecture patches

#
# x86(-64)
#
# Compile 686 kernels tuned for Pentium4.
%patch200 -p1
# add vidfail capability;
# without this patch specifying a framebuffer on the kernel prompt would
# make the boot stop if there's no supported framebuffer device; this is bad
# for the installer cd that wants to automatically fall back to textmode
# in that case
%patch201 -p1
# EDAC support for K8
%patch202 -p1
# Suppress APIC errors on UP x86-64.
%patch203 -p1
%if %{includexen}
# Suppress APIC errors on UP x86-64 (xen bits)
%patch204 -p1
%endif
# Don't delete cpu_devs data to identify different x86 types in late_initcall
%patch205 -p1
# quirk for Siemens Nixdorf AG FSC Multiprocessor Interrupt Controller
%patch206 -p1
# Blacklist Dell Optiplex 320 from using the HPET
%patch207 -p1
# Add x86-64 PM_TRACE support.
%patch209 -p1

#
# PowerPC
#
# Alleviate G5 thermal shutdown problems
%patch300 -p1
# Ensure slab objects are aligned enough for a uint64_t (#235392)
%patch301 -p1
# Fix data exception problem
%patch302 -p1
# Temporary hack to work around GCC PR #25724 / #21237
%patch303 -p1
# Fix up ibmvscsi for combined pSeries/iSeries build
%patch304 -p1
# Move pmac_zilog to its newly-registered device number
%patch305 -p1
# Ensure initrd memory is reserved at boot
%patch306 -p1
%patch307 -p1
%patch308 -p1
%patch309 -p1

# uevent support for of_platform device
%patch310 -p1
%patch311 -p1
%patch312 -p1
%patch313 -p1

#powermac-generic-suspend-*
%patch330 -p1
%patch331 -p1
%patch332 -p1
%patch333 -p1

# Efika Ethernet
%patch340 -p1
%patch341 -p1

# PlayStation 3 support
%patch350 -p1
%patch351 -p1
%patch352 -p1
%patch353 -p1
%patch354 -p1
%patch355 -p1
%patch356 -p1
%patch357 -p1
%patch358 -p1
%patch359 -p1
%patch360 -p1
%patch361 -p1
%patch362 -p1

%patch370 -p1
%patch371 -p1
%patch372 -p1
%patch373 -p1
%patch374 -p1
%patch375 -p1
%patch376 -p1
%patch377 -p1

# S390

#
# Patches 800 through 899 are reserved for bugfixes to the core system
# and patches related to how RPMs are build
#


# This patch adds a "make nonint_oldconfig" which is non-interactive and
# also gives a list of missing options at the end. Useful for automated
# builds (as used in the buildsystem).
%patch800 -p1

# Exec shield
%patch810 -p1
%if %{includexen}
%patch811 -p1
%endif

#
# GPG signed kernel modules
#
%patch900 -p1
%patch901 -p1
%patch902 -p1
%patch903 -p1
%patch904 -p1
%patch905 -p1
%patch906 -p1


#
# Patches 1000 to 5000 are reserved for bugfixes to drivers and filesystems
#


# Various low-impact patches to aid debugging.
%patch1010 -p1
%patch1011 -p1
%patch1012 -p1
%patch1013 -p1
%patch1015 -p1
%patch1016 -p1
%patch1019 -p1
# Disable the 'quiet' boot switch for better bug reports.
%patch1020 -p1
%patch1021 -p1
%patch1022 -p1
%if %{includexen}
%patch1023 -p1
%endif

#
# Make /dev/mem a need-to-know function
#
%patch1050 -p1
%if %{includexen}
%patch1051 -p1
%endif

#
# /dev/crash driver for the crashdump analysis tool
#
%patch1060 -p1
%if %{includexen}
%patch1061 -p1
%endif

#
# SCSI Bits.
#
# Fix old SCSI adapter crashes with CD-ROM
%patch1100 -p1
# fix cpqarray pci enable
%patch1106 -p1

# Fix 3w-9xxx to use PCI resource length correctly (bug #309611)
%patch1107 -p1

#
# Various NFS/NFSD fixes.
#

# NIC driver fixes
# Don't print warnings about MSI failures on e1000
%patch1300 -p1

# Filesystem patches.
# Squashfs
%patch1400 -p1
# GFS2 update
%patch1410 -p1

# Networking
# Disable easy to trigger printk's.
%patch1500 -p1

# Misc fixes
# Add missing MODULE_VERSION tags to some modules.
%patch1600 -p1
# Fix SHA1 alignment problem on ia64
%patch1601 -p1
# The input layer spews crap no-one cares about.
%patch1610 -p1
# don't wakeup ondemand timer whilst idle.
%patch1620 -p1
# Update KVM. (should we just carry the latest version?)
%patch1630 -p1
# KVM: Detect if AMD svm was disabled by BIOS
%patch1631 -p1
# KVM: Fix above patch on 32-bit
%patch1632 -p1
# KVM: reinit real-mode TSS before switching back
%patch1633 -p1
# Allow to use 480600 baud on 16C950 UARTs
%patch1650 -p1
# Allow large files on UDF
%patch1660 -p1
%patch1661 -p1
%patch1662 -p1
# fix oops in sysfs_readdir
%patch1670 -p1

%if %{includexen}
# checksum fixes (bug #223258)
%patch1665 -p1
%endif

# Align kernel data segment to page boundary.
%patch1690 -p1
# DVB spinlock bug
%patch1700 -p1
# NFS: Added support to turn off the NFSv3 READDIRPLUS RPC.
%patch1710 -p1
# Missing braces
%patch1711 -p1
# setuid /proc/self/maps fix.
%patch1720 -p1
# Fix ACPI suspend / device suspend ordering problem
%patch1730 -p1
# Add a safety net to softlockup so that it doesn't prevent installs.
%patch1740 -p1
# Speed up spinlock debug.
%patch1770 -p1
# Silence some useless messages that still get printed with 'quiet'
%patch1771 -p1
# softirqs: print out irq-trace events
%patch1781 -p1

# OLPC specific patches
%if 0%{?olpc}
# Marvell Libertas wireless driver
%patch1791 -p1
# OLPC touchpad
%patch1792 -p1
%endif

# temporarily restore START_ARRAY ioctl
%patch1793 -p1

# Fix i82875 EDAC driver setup so X will start
%patch1794 -p1
# Work around sysfs/uevent use-after-free problems with Bluetooth HID
%patch1795 -p1

%if %{includexen}
# Xenoprof's DOMAIN_SWITCH_CODE changed in oprofile 0.9.3
%patch1796 -p1
%endif

# Fix the SELinux mprotect checks on executable mappings
%patch1801 -p1

# Remove kernel-internal functionality that nothing external should use.
%patch1910 -p1

#
# VM related fixes.
#
# Re-add cond_resched to invalidate_mapping_pages()
%patch2000 -p1
# Silence GFP_ATOMIC failures.
%patch2001 -p1

# Changes to upstream defaults.
# Use UTF-8 by default on VFAT.
%patch2100 -p1

# Use unicode VT's by default.
%patch2103 -p1
# Disable NMI watchdog by default.
%patch2105 -p1
# Disable MMCONFIG & MSI by default.
%patch2106 -p1
# Disable USB autosuspend by default.
%patch2107 -p1

# HPA support for libata.
%patch2201 -p1
# sata_nv: Don't attempt using ADMA for (READ|SET)_MAX commands
%patch2202 -p1
# Disable ATAPI DMA on ALI chipsets.
%patch2203 -p1
# libata: don't initialize sg in ata_exec_internal() if DMA_NONE
# ia64 ata quirk
%patch2204 -p1
# remove the wildcard from sata_nv driver
%patch2206 -p1
# pata_pcmcia.c: add card ident for jvc cdrom
%patch2207 -p1
# Add libata ID's for ATI SB700
%patch2208 -p1
# hpt3x2n: Correct revision boundary
%patch2209 -p1
# pata_sis: Fix and clean up some timing setups
%patch2210 -p1
# libata: always use polling SETXFER
%patch2211 -p1
# pata_ali: limit DMA speeds
%patch2212 -p1
# libata: use PIO when xfer is not a multiple of 16
%patch2213 -p1
# libata: call check_dma with qc better prepared
%patch2214 -p1
# libata: add intel ich8m (santa rosa) pata ID
%patch2215 -p1
# libata: add option to disable PATA DMA
%patch2216 -p1
# libata: update NCQ blacklist
%patch2217 -p1
# libata: partially fix dma for pata_it821x
%patch2218 -p1

# ATA spindown
%patch2220 -p1
%patch2221 -p1
%patch2222 -p1
%patch2223 -p1
%patch2224 -p1
%patch2225 -p1
%patch2226 -p1
%patch2227 -p1
%patch2228 -p1

# Add critical wireless updates from 2.6.22
%patch2300 -p1
# Add the new wireless stack and drivers from wireless-dev
%patch2301 -p1
# avoid bcm43xx vs bcm43xx-mac80211 PCI ID conflicts
%patch2302 -p1

# Assorted dyntick/clock/timer fixes.
%patch2402 -p1
%patch2403 -p1


# ACPI patches
# Remove Dell Optiplex GX240 from the ACPI blacklist
%patch2500 -p1
# Fix ACPI dock oops (#238054)
%patch2501 -p1
# Fix another ACPI boot regression.
%patch2502 -p1
# Fix possible breakage of ACPI suspend
%patch2503 -p1

# Fix excessive wakeups
# Make hdaps timer only tick when in use.
%patch2600 -p1


# ACPI patches
%patch3000 -p1

#
# Patches 5000 to 6000 are reserved for new drivers that are about to
# be merged upstream
#

# Pull in the new firewire stack from 2.6.20-rc3-mm1.
%patch5000 -p1
%patch5001 -p1

#
# final stuff
#

#
# misc small stuff to make things compile or otherwise improve performance
#

# END OF PATCH APPLICATIONS


%endif

# Any further pre-build tree manipulations happen here.

if [ -e scripts/checkpatch.pl ];then
  chmod +x scripts/checkpatch.pl
fi

cp %{SOURCE10} Documentation/

mkdir configs

# Remove configs not for the buildarch
for cfg in kernel-%{version}-*.config; do
  if [ `echo %{all_arch_configs} | grep -c $cfg` -eq 0 ]; then
    rm -f $cfg
  fi
done

%if !%{with_debug}
rm -f kernel-%{version}-*-debug.config
%endif

# now run oldconfig over all the config files
for i in *.config
do
  mv $i .config
  Arch=`head -1 .config | cut -b 3-`
  make ARCH=$Arch %{oldconfig_target} > /dev/null
  echo "# $Arch" > configs/$i
  cat .config >> configs/$i
done

# get rid of unwanted files resulting from patch fuzz
find . \( -name "*.orig" -o -name "*~" \) -exec rm -f {} \; >/dev/null

cd ..

# Unpack the Xen tarball.
%if %{includexen}
cp %{SOURCE2} .
if [ -d xen ]; then
  rm -rf xen
fi
%setup -D -T -q -n kernel-%{kversion} -a1
cd %{xen_hv_dirname}/
# Any necessary hypervisor patches go here
%patch20000 -p1
%patch20001 -p1
%patch20004 -p1
%patch20005 -p1

%endif


###
### build
###
%build

%if %{usesparse}
%define sparse_mflags	C=1
%endif

%if %{fancy_debuginfo}
# This override tweaks the kernel makefiles so that we run debugedit on an
# object before embedding it.  When we later run find-debuginfo.sh, it will
# run debugedit again.  The edits it does change the build ID bits embedded
# in the stripped object, but repeating debugedit is a no-op.  We do it
# beforehand to get the proper final build ID bits into the embedded image.
# This affects the vDSO images in vmlinux, and the vmlinux image in bzImage.
idhack='cmd_objcopy=$(if $(filter -S,$(OBJCOPYFLAGS)),'\
'sh -xc "/usr/lib/rpm/debugedit -b $$RPM_BUILD_DIR -d /usr/src/debug -i $<";)'\
'$(OBJCOPY) $(OBJCOPYFLAGS) $(OBJCOPYFLAGS_$(@F)) $< $@'
%endif

BuildKernel() {
    MakeTarget=$1
    KernelImage=$2
    Flavour=$3

    # Pick the right config file for the kernel we're building
    if [ -n "$Flavour" ] ; then
      Config=kernel-%{version}-%{_target_cpu}-$Flavour.config
      DevelDir=/usr/src/kernels/%{KVERREL}-$Flavour-%{_target_cpu}
      DevelLink=/usr/src/kernels/%{KVERREL}$Flavour-%{_target_cpu}
    else
      Config=kernel-%{version}-%{_target_cpu}.config
      DevelDir=/usr/src/kernels/%{KVERREL}-%{_target_cpu}
      DevelLink=
    fi

    KernelVer=%{version}-%{release}$Flavour
    echo BUILDING A KERNEL FOR $Flavour %{_target_cpu}...

    # make sure EXTRAVERSION says what we want it to say
    perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = %{?stablerev}-%{release}$Flavour/" Makefile

    # if pre-rc1 devel kernel, must fix up SUBLEVEL for our versioning scheme
    %if !0%{?rcrev}
    %if 0%{?gitrev}
    perl -p -i -e 's/^SUBLEVEL.*/SUBLEVEL = %{upstream_sublevel}/' Makefile
    %endif
    %endif

    # and now to start the build process

    make -s mrproper
    cp configs/$Config .config

    Arch=`head -1 .config | cut -b 3-`
    echo USING ARCH=$Arch

    make -s ARCH=$Arch %{oldconfig_target} > /dev/null
    make -s ARCH=$Arch %{?_smp_mflags} $MakeTarget %{?sparse_mflags} \
    	 ${idhack+"$idhack"}
    make -s ARCH=$Arch %{?_smp_mflags} modules %{?sparse_mflags} || exit 1

    # Start installing the results
%if %{with_debuginfo}
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/boot
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/%{image_install_path}
%endif
    mkdir -p $RPM_BUILD_ROOT/%{image_install_path}
    install -m 644 .config $RPM_BUILD_ROOT/boot/config-$KernelVer
    install -m 644 System.map $RPM_BUILD_ROOT/boot/System.map-$KernelVer
    touch $RPM_BUILD_ROOT/boot/initrd-$KernelVer.img
    cp $KernelImage $RPM_BUILD_ROOT/%{image_install_path}/vmlinuz-$KernelVer
    if [ -f arch/$Arch/boot/zImage.stub ]; then
      cp arch/$Arch/boot/zImage.stub $RPM_BUILD_ROOT/%{image_install_path}/zImage.stub-$KernelVer || :
    fi

    if [ "$Flavour" == "kdump" ]; then
        cp vmlinux $RPM_BUILD_ROOT/%{image_install_path}/vmlinux-$KernelVer
        rm -f $RPM_BUILD_ROOT/%{image_install_path}/vmlinuz-$KernelVer
    fi

    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer
    make -s ARCH=$Arch INSTALL_MOD_PATH=$RPM_BUILD_ROOT modules_install KERNELRELEASE=$KernelVer
%if %{do_vdso_install}
%ifarch %{vdso_arches}
    make -s ARCH=$Arch INSTALL_MOD_PATH=$RPM_BUILD_ROOT vdso_install KERNELRELEASE=$KernelVer
%endif
%endif

    # And save the headers/makefiles etc for building modules against
    #
    # This all looks scary, but the end result is supposed to be:
    # * all arch relevant include/ files
    # * all Makefile/Kconfig files
    # * all script/ files

    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/source
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    (cd $RPM_BUILD_ROOT/lib/modules/$KernelVer ; ln -s build source)
    # dirs for additional modules per module-init-tools, kbuild/modules.txt
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/extra
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/updates
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/weak-updates
    # first copy everything
    cp --parents `find  -type f -name "Makefile*" -o -name "Kconfig*"` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp Module.symvers $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    # then drop all but the needed Makefiles/Kconfig files
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Documentation
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts
    rm -rf $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cp .config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    cp -a scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    if [ -d arch/%{_arch}/scripts ]; then
      cp -a arch/%{_arch}/scripts $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch} || :
    fi
    if [ -f arch/%{_arch}/*lds ]; then
      cp -a arch/%{_arch}/*lds $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/%{_arch}/ || :
    fi
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*.o
    rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/scripts/*/*.o
    mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cd include
    cp -a acpi config keys linux math-emu media mtd net pcmcia rdma rxrpc scsi sound video asm asm-generic $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    cp -a `readlink asm` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    if [ "$Arch" = "x86_64" ]; then
      cp -a asm-i386 $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
    fi
    # While arch/powerpc/include/asm is still a symlink to the old
    # include/asm-ppc{64,} directory, include that in kernel-devel too.
    if [ "$Arch" = "powerpc" -a -r ../arch/powerpc/include/asm ]; then
      cp -a `readlink ../arch/powerpc/include/asm` $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
      mkdir -p $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/$Arch/include
      pushd $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/arch/$Arch/include
      ln -sf ../../../include/asm-ppc* asm
      popd
    fi
%if %{includexen}
    cp -a xen $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include
%endif

    # Make sure the Makefile and version.h have a matching timestamp so that
    # external modules can be built
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/Makefile $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/version.h
    touch -r $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/linux/autoconf.h
    # Copy .config to include/config/auto.conf so "make prepare" is unnecessary.
    cp $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/.config $RPM_BUILD_ROOT/lib/modules/$KernelVer/build/include/config/auto.conf
    cd ..

    #
    # save the vmlinux file for kernel debugging into the kernel-debuginfo rpm
    #
%if %{with_debuginfo}
    mkdir -p $RPM_BUILD_ROOT%{debuginfodir}/lib/modules/$KernelVer
    cp vmlinux $RPM_BUILD_ROOT%{debuginfodir}/lib/modules/$KernelVer
%endif

    find $RPM_BUILD_ROOT/lib/modules/$KernelVer -name "*.ko" -type f >modnames

    # mark modules executable so that strip-to-file can strip them
    xargs --no-run-if-empty chmod u+x < modnames

    # Generate a list of modules for block and networking.

    fgrep /drivers/ modnames | xargs --no-run-if-empty nm -upA |
    sed -n 's,^.*/\([^/]*\.ko\):  *U \(.*\)$,\1 \2,p' > drivers.undef

    collect_modules_list()
    {
      sed -r -n -e "s/^([^ ]+) \\.?($2)\$/\\1/p" drivers.undef |
      LC_ALL=C sort -u > $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.$1
    }

    collect_modules_list networking \
    			 'register_netdev|ieee80211_register_hw|usbnet_probe'
    collect_modules_list block \
    			 'ata_scsi_ioctl|scsi_add_host|blk_init_queue'

    # detect missing or incorrect license tags
    rm -f modinfo
    while read i
    do
      echo -n "${i#$RPM_BUILD_ROOT/lib/modules/$KernelVer/} " >> modinfo
      /sbin/modinfo -l $i >> modinfo
    done < modnames

    egrep -v \
    	  'GPL( v2)?$|Dual BSD/GPL$|Dual MPL/GPL$|GPL and additional rights$' \
	  modinfo && exit 1

    rm -f modinfo modnames

    # remove files that will be auto generated by depmod at rpm -i time
    for i in alias ccwmap dep ieee1394map inputmap isapnpmap ofmap pcimap seriomap symbols usbmap
    do
      rm -f $RPM_BUILD_ROOT/lib/modules/$KernelVer/modules.$i
    done

    # Move the devel headers out of the root file system
    mkdir -p $RPM_BUILD_ROOT/usr/src/kernels
    mv $RPM_BUILD_ROOT/lib/modules/$KernelVer/build $RPM_BUILD_ROOT/$DevelDir
    ln -sf ../../..$DevelDir $RPM_BUILD_ROOT/lib/modules/$KernelVer/build
    [ -z "$DevelLink" ] || ln -sf `basename $DevelDir` $RPM_BUILD_ROOT/$DevelLink
}

###
# DO it...
###

# prepare directories
rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/boot

%if %{includexen}
%if %{with_xen}
  cd %{xen_hv_dirname}/xen/
  mkdir -p $RPM_BUILD_ROOT/%{image_install_path} $RPM_BUILD_ROOT/boot
  make %{?_smp_mflags} %{xen_flags}
  install -m 644 xen.gz $RPM_BUILD_ROOT/%{image_install_path}/xen.gz-%{KVERREL}
  install -m 755 xen-syms $RPM_BUILD_ROOT/boot/xen-syms-%{KVERREL}
  cd ../..
%endif
%endif

cd linux-%{kversion}.%{_target_cpu}

%if %{with_debug}
BuildKernel %make_target %kernel_image debug
%if %{with_pae}
BuildKernel %make_target %kernel_image PAE-debug
%endif
%endif

%if %{with_pae}
BuildKernel %make_target %kernel_image PAE
%endif

%if %{with_up}
BuildKernel %make_target %kernel_image
%endif

%if %{with_smp}
BuildKernel %make_target %kernel_image smp
%endif

%if %{includexen}
%if %{with_xen}
BuildKernel %xen_target %xen_image xen
%endif
%endif

%if %{with_kdump}
BuildKernel %make_target %kernel_image kdump
%endif

###
### Special hacks for debuginfo subpackages.
###

# This macro is used by %%install, so we must redefine it before that.
%define debug_package %{nil}

%if %{fancy_debuginfo}
%define __debug_install_post \
  /usr/lib/rpm/find-debuginfo.sh %{debuginfo_args} %{_builddir}/%{?buildsubdir}\
%{nil}
%endif

%if %{with_debuginfo}
%ifnarch noarch
%global __debug_package 1
%files -f debugfiles.list -n %{mainpkgname}-debuginfo-common
%defattr(-,root,root)
%endif
%endif

###
### install
###

%install

cd linux-%{kversion}.%{_target_cpu}

%if %{includexen}
%if %{with_xen}
mkdir -p $RPM_BUILD_ROOT/etc/ld.so.conf.d
rm -f $RPM_BUILD_ROOT/etc/ld.so.conf.d/kernelcap-%{KVERREL}.conf
cat > $RPM_BUILD_ROOT/etc/ld.so.conf.d/kernelcap-%{KVERREL}.conf <<\EOF
# This directive teaches ldconfig to search in nosegneg subdirectories
# and cache the DSOs there with extra bit 0 set in their hwcap match
# fields.  In Xen guest kernels, the vDSO tells the dynamic linker to
# search in nosegneg subdirectories and to match this extra hwcap bit
# in the ld.so.cache file.
hwcap 0 nosegneg
EOF
chmod 444 $RPM_BUILD_ROOT/etc/ld.so.conf.d/kernelcap-%{KVERREL}.conf
%endif
%endif

%if %{with_doc}
mkdir -p $RPM_BUILD_ROOT/usr/share/doc/kernel-doc-%{kversion}/Documentation

# sometimes non-world-readable files sneak into the kernel source tree
chmod -R a+r *
# copy the source over
tar cf - Documentation | tar xf - -C $RPM_BUILD_ROOT/usr/share/doc/kernel-doc-%{kversion}
%endif

%if %{with_headers}
# Install kernel headers
make ARCH=%{hdrarch} INSTALL_HDR_PATH=$RPM_BUILD_ROOT/usr headers_install

# Manually go through the 'headers_check' process for every file, but
# don't die if it fails
chmod +x scripts/hdrcheck.sh
echo -e '*****\n*****\nHEADER EXPORT WARNINGS:\n*****' > hdrwarnings.txt
for FILE in `find $RPM_BUILD_ROOT/usr/include` ; do
    scripts/hdrcheck.sh $RPM_BUILD_ROOT/usr/include $FILE /dev/null >> hdrwarnings.txt || :
done
echo -e '*****\n*****' >> hdrwarnings.txt
if grep -q exist hdrwarnings.txt; then
   sed s:^$RPM_BUILD_ROOT/usr/include/:: hdrwarnings.txt
   # Temporarily cause a build failure if header inconsistencies.
   # exit 1
fi

# glibc provides scsi headers for itself, for now
rm -rf $RPM_BUILD_ROOT/usr/include/scsi
rm -f $RPM_BUILD_ROOT/usr/include/asm*/atomic.h
rm -f $RPM_BUILD_ROOT/usr/include/asm*/io.h
rm -f $RPM_BUILD_ROOT/usr/include/asm*/irq.h
%endif

###
### clean
###

%clean
rm -rf $RPM_BUILD_ROOT

###
### scripts
###

#
# This macro defines a %%post script for a kernel*-devel package.
#	%%kernel_devel_post <subpackage>
#
%define kernel_devel_post() \
%{expand:%%post -n %{variantbase}-%{?1:%{1}-}devel}\
if [ -f /etc/sysconfig/kernel ]\
then\
    . /etc/sysconfig/kernel || exit $?\
fi\
if [ "$HARDLINK" != "no" -a -x /usr/sbin/hardlink ]\
then\
    (cd /usr/src/kernels/%{KVERREL}-%{?1:%{1}-}%{_target_cpu} &&\
     /usr/bin/find . -type f | while read f; do\
       hardlink -c /usr/src/kernels/*.fc*-*/$f $f\
     done)\
fi\
%{nil}

#
# This macro defines a %%post script for a kernel package and its devel package.
#	%%kernel_variant_post [-v <subpackage>] [-s <s> -r <r>] <mkinitrd-args>
# More text can follow to go at the end of this variant's %%post.
#
%define kernel_variant_post(s:r:v:) \
%{expand:%%kernel_devel_post %{?-v*}}\
%{expand:%%post -n %{variantbase}%{?-v:-%{-v*}}}\
%{-s:\
if [ `uname -i` == "x86_64" -o `uname -i` == "i386" ] &&\
   [ -f /etc/sysconfig/kernel ]; then\
  /bin/sed -i -e 's/^DEFAULTKERNEL=%{-s*}$/DEFAULTKERNEL=%{-r*}/' /etc/sysconfig/kernel || exit $?\
fi}\
/sbin/new-kernel-pkg --package kernel%{?-v:-%{-v*}} --mkinitrd --depmod --install %{*} %{KVERREL}%{?-v*} || exit $?\
#if [ -x /sbin/weak-modules ]\
#then\
#    /sbin/weak-modules --add-kernel %{KVERREL}%{?-v*} || exit $?\
#fi\
%{nil}

#
# This macro defines a %%preun script for a kernel package.
#	%%kernel_variant_preun <subpackage>
#
%define kernel_variant_preun() \
%{expand:%%preun -n %{variantbase}%{?1:-%{1}}}\
/sbin/new-kernel-pkg --rminitrd --rmmoddep --remove %{KVERREL}%{?1} || exit $?\
#if [ -x /sbin/weak-modules ]\
#then\
#    /sbin/weak-modules --remove-kernel %{KVERREL}%{?1} || exit $?\
#fi\
%{nil}

%if %{with_up}
%kernel_variant_preun
%kernel_variant_post -s kernel-smp -r kernel
%endif

%if %{with_smp}
%kernel_variant_preun smp
%kernel_variant_post -v smp
%endif

%if %{with_pae}
%kernel_variant_preun PAE
%kernel_variant_post -v PAE -s kernel-smp -r kernel-PAE
%endif

%if %{with_debug}
%kernel_variant_preun debug
%kernel_variant_post -v debug
%endif

%if %{with_pae_debug}
%kernel_variant_preun PAE-debug
%kernel_variant_post -v PAE-debug -s kernel-smp -r kernel-PAE-debug
%endif

%if %{with_xen}
%kernel_variant_preun xen
%kernel_variant_post -v xen -s kernel-xen[0U] -r kernel-xen -- `[ -d /proc/xen -a ! -e /proc/xen/xsd_kva ] || echo --multiboot=/%{image_install_path}/xen.gz-%{KVERREL}`
if [ -x /sbin/ldconfig ]
then
    /sbin/ldconfig -X || exit $?
fi
%endif

###
### file lists
###

%if %{with_headers}
%files -n %{mainpkgname}-headers
%defattr(-,root,root)
/usr/include/*
%endif

# only some architecture builds need kernel-doc
%if %{with_doc}
%files -n %{mainpkgname}-doc
%defattr(-,root,root)
%{_datadir}/doc/kernel-doc-%{kversion}/Documentation/*
%dir %{_datadir}/doc/kernel-doc-%{kversion}/Documentation
%dir %{_datadir}/doc/kernel-doc-%{kversion}
%endif

# This is %{image_install_path} on an arch where that includes ELF files,
# or empty otherwise.
%define elf_image_install_path %{?kernel_image_elf:%{image_install_path}}

#
# This macro defines the %%files sections for a kernel package
# and its devel and debuginfo packages.
#	%%kernel_variant_files [-k vmlinux] [-a <extra-files-glob>] [-e <extra-nonbinary>] <condition> <subpackage>
#
%define kernel_variant_files(a:e:k:) \
%if %{1}\
%{expand:%%files -n %{variantbase}%{?2:-%{2}}}\
%defattr(-,root,root)\
/%{image_install_path}/%{?-k:%{-k*}}%{!?-k:vmlinuz}-%{KVERREL}%{?2}\
/boot/System.map-%{KVERREL}%{?2}\
#/boot/symvers-%{KVERREL}%{?2}.gz\
/boot/config-%{KVERREL}%{?2}\
%{?-a:%{-a*}}\
%dir /lib/modules/%{KVERREL}%{?2}\
/lib/modules/%{KVERREL}%{?2}/kernel\
/lib/modules/%{KVERREL}%{?2}/build\
/lib/modules/%{KVERREL}%{?2}/source\
/lib/modules/%{KVERREL}%{?2}/extra\
/lib/modules/%{KVERREL}%{?2}/updates\
/lib/modules/%{KVERREL}%{?2}/weak-updates\
%if %{do_vdso_install}\
%ifarch %{vdso_arches}\
/lib/modules/%{KVERREL}%{?2}/vdso\
%endif\
%endif\
/lib/modules/%{KVERREL}%{?2}/modules.block\
/lib/modules/%{KVERREL}%{?2}/modules.networking\
%ghost /boot/initrd-%{KVERREL}%{?2}.img\
%{?-e:%{-e*}}\
%{expand:%%files -n %{variantbase}-%{?2:%{2}-}devel}\
%defattr(-,root,root)\
%verify(not mtime) /usr/src/kernels/%{KVERREL}%{?2:-%{2}}-%{_target_cpu}\
/usr/src/kernels/%{KVERREL}%{?2}-%{_target_cpu}\
%if %{with_debuginfo}\
%ifnarch noarch\
%if %{fancy_debuginfo}\
%{expand:%%files -f debuginfo%{?2}.list -n %{variantbase}-%{?2:%{2}-}debuginfo}\
%else\
%{expand:%%files -n %{variantbase}-%{?2:%{2}-}debuginfo}\
%endif\
%defattr(-,root,root)\
%if !%{fancy_debuginfo}\
%if "%{elf_image_install_path}" != ""\
%{debuginfodir}/%{elf_image_install_path}/*-%{KVERREL}%{?2}.debug\
%endif\
%{debuginfodir}/lib/modules/%{KVERREL}%{?2}\
%{debuginfodir}/usr/src/kernels/%{KVERREL}%{?2:-%{2}}-%{_target_cpu}\
%endif\
%endif\
%endif\
%endif\
%{nil}


%kernel_variant_files %{with_up}
%kernel_variant_files %{with_smp} smp
%kernel_variant_files %{with_debug} debug
%kernel_variant_files %{with_pae} PAE
%kernel_variant_files %{with_pae_debug} PAE-debug
%kernel_variant_files -k vmlinux %{with_kdump} kdump
%kernel_variant_files -a /%{image_install_path}/xen*-%{KVERREL} -e /etc/ld.so.conf.d/kernelcap-%{KVERREL}.conf %{with_xen} xen

%changelog
* Fri Jan 18 2008 Daniel P. Berrange <berrange@redhat.com>
- Update to 3.2.0 final release

* Fri Jan 11 2008 Daniel P. Berrange <berrange@redhat.com>
- Rebuild due to koji build system failure

* Thu Jan 10 2008 Daniel P. Berrange <berrange@redhat.com>
- Update HV to 3.2.0 rc5 changeset 16701

* Mon Nov 19 2007 Eduardo Habkost <ehabkost@redhat.com>
- Re-add xen-hypervisor-abi provides. It was lost on the spec resync
  (problem detected on updates-testing)

* Wed Nov 14 2007 Eduardo Habkost <ehabkost@redhat.com>
- Update HV to official 3.1.0

* Mon Oct 29 2007  Eduardo Habkost <ehabkost@redhat.com>
- Pulled spec file from non-xen kernel
  - Enabled -debuginfo packages (bug #336351)

* Wed Oct 10 2007 Eduardo Habkost <ehabkost@redhat.com>
- Remove bogus WARN_ON() from synchronize_irq() (bug #293451)

* Mon Oct 08 2007 Eduardo Habkost <ehabkost@redhat.com>
- Add patch to fix BUG_ON() Oops caused by privcmd (bug #323031)

* Thu Sep 27 2007 Eduardo Habkost <ehabkost@redhat.com>
- Fix 3w-9xxx to use PCI resources properly and work under Xen (bug #309611)

* Wed Sep 26 2007 Eduardo Habkost <ehabkost@redhat.com>
- Disable Firewire on i686. It is not supported by the current
  Xen swiotlb implementation (bug #307461)

* Thu Sep 20 2007 Eduardo Habkost <ehabkost@redhat.com>
- Fix sleazy-fpu implementation to actually avoid on fpu-intensive
  programs

* Thu Sep 20 2007 Eduardo Habkost <ehabkost@redhat.com>
- Add fix for bug #294011 (floating point corruption)
  (changeset 8c24767501ff from Xen 3.1.1)

* Thu Sep 13 2007 Eduardo Habkost <ehabkost@redhat.com>
- Rebase to 2.6.21.7

* Thu Sep 13 2007 Eduardo Habkost <ehabkost@redhat.com>
- Fix for bug #248398: HVM domain hangs during (F-7) install.
  Backported fix from xen-unstable: 15239:656b8175f4f2

* Thu Sep 13 2007 Eduardo Habkost <ehabkost@redhat.com>
- Make hypervisor patches relative to the xen HG tree, not only the
  hypervisor subdirectory. This makes easier to import patches from
  upstream.

* Wed Sep 12 2007 Eduardo Habkost <ehabkost@redhat.com>
- Disable CONFIG_HIGHPTE. It is not reliable and cause performance loss
  under Xen.

* Thu Sep 06 2007 Eduardo Habkost <ehabkost@redhat.com>
- Include modules.block and modules.networking on the package (bug #280731)

* Thu Aug 30 2007 Eduardo Habkost <ehabkost@redhat.com>
- Provides: xen-hypervisor-abi = 3.1 (bug #269581)

* Wed Aug 29 2007 Eduardo Habkost <ehabkost@redhat.com>
- BuildRequires: /bin/awk. It is needed by Xen and not included
  on the default Koji buildroot anymore

* Mon Aug 27 2007 Eduardo Habkost <ehabkost@redhat.com>
- Disabled KVM on config files. It wouldn't work on kernel-xen and
  causes crashes when loaded

* Mon Aug 20 2007 Eduardo Habkost <ehabkost@redhat.com>
- Use %%if %%{includexen} around the xen-specific patches
- Add linux-2.6-xen-irq_vector-uninitialize.patch:
  Fix Xen IRQ handling bug that was introduced on the rebase
  to 2.6.21. It may fix some problems reported on bug #252301

* Thu Aug 09 2007 Eduardo Habkost <ehabkost@redhat.com>
- Add linux-2.6-xen-backwards-time.patch from linux-2.6.18-xen.hg
  changeset 87bb8705768a. This should fix bug #236307

* Tue Aug 07 2007 Eduardo Habkost <ehabkost@redhat.com>
- Re-add linux-2.6-disable-netback-checksum.patch. The other checksum
  patches are already included on linux-2.6-xen-3.1.0.patch.

* Mon Aug 06 2007 Eduardo Habkost <ehabkost@redhat.com>
- Xenoprof: fix DOMAIN_SWITCH_CODE value according to oprofile 0.9.3
  (Markus Armbruster)

* Tue Jul 24 2007 Eduardo Habkost <ehabkost@redhat.com>
- Rebase to 2.6.21.6

* Wed Jul 20 2007 Eduardo Habkost <ehabkost@redhat.com>
- Update xen patch to xen 3.1.0
- Remove old network checksum patches. They should be
  already included in the xen patch

* Tue Jul 17 2007 Eduardo Habkost <ehabkost@redhat.com>
- Make blkfront module wait for the disk to be added (bug #248462)
- Rebase to 2.6.20.14

* Thu Jul 12 2007 Eduardo Habkost <ehabkost@redhat.com>
- Remove #ifdef CONFIG_PM from acpi poweroff code (fix bug #241381)
- BuildRequires: python for xen buildsystem mkheader.py

* Mon Jun 11 2007 Eduardo Habkost <ehabkost@redhat.com>
- Added fix to bug #215201: linux-2.6-xen-fix-nosegneg-detection.patch
- Disabled CONFIG_PROVE_LOCKING, as the xen code doesn't support it.
  See bug #239601

* Tue May 22 2007 Eduardo Habkost <ehabkost@redhat.com>
- Add patch from bug #209429: linux-2.6-xen-iscsi-x86_64-no_iommu_init.patch

* Thu May 10 2007  <quintela@redhat.com>
- sync irq/manage.c with upstream (Markus).
- correctly init smpboot.c for xen (Chris Wright).
- pop %gs is a 2 bytes opcode (sct).

* Tue May  8 2007  <quintela@redhat.com>
- Disable kvm on xen kernels

* Fri May 04 2007 Eduardo Habkost <ehabkost@redhat.com>
- Updated Xen Hypervisor tarball to 3.1.0-rc7, hg cset 7041b52471c3

* Tue May 01 2007 Eduardo Habkost <ehabkost@redhat.com>
- Updated Xen Hypervisor to 3.0.5-rc4, hg cset 1668299c0ea4
- Added smp and vgetcpu fixes (bug #238015)
- Added patch from Heberter Xu, disabling checksum on netback,
  to work around problems with DHCP and partial checksums on guests

* Fri Apr 27 2007 Eduardo Habkost <ehabkost@redhat.com>
- Updated Xen Hypervisor to 3.0.5-rc3, hg cset 14934
- Included checksum patches to fix bug #223258

* Thu Mar 22 2007 Juan Quintela <quintela@redhat.com>
- disable alternative firewire stack for x86_64, it hangs kernel
  during boot.

* Thu Mar 22 2007 Juan Quintela <quintela@redhat.com>
- Add missing git-geode.patch.
- Re-enable Xen and update to 2.6.20-3.0.4 version.
- Update to 2.6.20.3.

* Mon Feb 12 2007 Kristian Høgsberg <krh@redhat.com>
- Update firewire patch with latest usptream changes.

* Mon Feb  5 2007 Dave Jones <davej@redhat.com>
- Fix attr2 corruption with btree data extents

* Sun Feb  4 2007 Dave Jones <davej@redhat.com>
- 2.6.20
- Move xen sources out of kernel-xen-devel. (Don Zickus)

* Sat Feb  3 2007 Dave Jones <davej@redhat.com>
- 2.6.20-rc7-git1

* Wed Jan 31 2007 Dave Jones <davej@redhat.com>
- 2.6.20rc7

* Tue Jan 30 2007 Dave Jones <davej@redhat.com>
- 2.6.20rc6-git2

* Mon Jan 29 2007 Chuck Ebbert <cebbert@redhat.com>
- 2.6.20rc6-git1

* Fri Jan 26 2007 Bill Nottingham <notting@redhat.com>
- turn on CONFIG_SYSFS_DEPRECATED so that things actually work. *sigh*

* Tue Jan 23 2007 Jeremy Katz <katzj@redhat.com>
- temporarily restore START_ARRAY ioctl for mdraid so that we can have mdraid
  in test1

* Sun Jan 14 2007 Roland McGrath <roland@redhat.com>
- utrace update: PTRACE_TRACEME fix

* Thu Jan 11 2007 Kristian Høgsberg <krh@redhat.com>
- Add and enable alternative firewire stack.

* Thu Jan 11 2007 Jeremy Katz <katzj@redhat.com>
- disable nmi watchdog by default

* Wed Jan 10 2007 Dave Jones <davej@redhat.com>
- 2.6.20rc4-git4
- PIIX3 PATA support.

* Sun Jan  7 2007 Dave Jones <davej@redhat.com>
- 2.6.20rc4

* Sat Jan  6 2007 Dave Jones <davej@redhat.com>
- 2.6.20rc3-git4

* Mon Jan  1 2007 Dave Jones <davej@redhat.com>
- 2.6.20rc3

* Mon Jan  1 2007 David Woodhouse <dwmw2@redhat.com>
- Fix fallout from half-baked upstream IBM vSCSI patch

* Mon Jan  1 2007 David Woodhouse <dwmw2@redhat.com>
- Fix BUG() in assembler in PowerPC
- Fix Maple IDE IRQ

* Sun Dec 31 2006 Dave Jones <davej@redhat.com>
- 2.6.20rc2-git2

* Fri Dec 29 2006 Dave Jones <davej@redhat.com>
- 2.6.20rc2-git1

* Fri Dec 29 2006 David Woodhouse <dwmw2@redhat.com>
- Update USB endianness patches

* Wed Dec 27 2006 Dave Jones <davej@redhat.com>
- 2.6.20rc2

* Thu Dec 21 2006 David Woodhouse <dwmw2@redhat.com>
- Fix IPv6 checksum handling
- Add MODULE_ALIAS() to pata_mpc52xx driver

* Wed Dec 20 2006 David Woodhouse <dwmw2@redhat.com>
- Fix BE OHCI support -- write only BE not BE and _then_ LE. Doh.

* Tue Dec 19 2006 David Woodhouse <dwmw2@redhat.com>
- Fix ebus oops
- Actually make it possible to include iSeries in the generic PPC64 kernel

* Tue Dec 19 2006 David Woodhouse <dwmw2@redhat.com>
- Include iSeries in the generic PPC64 kernel now that's possible
- Fix squashfs
- Fix softmac fallout from work_struct changes
- Update OF uevent handler patch

* Mon Dec 18 2006 David Woodhouse <dwmw2@redhat.com>
- Enable CONFIG_IDE on PowerPC, for PMAC. Need pata_pmac driver...
- Fix the deleteme handling
- Fix build on platforms without writel_be()
- Fix connector build failure

* Sun Dec 17 2006 Dave Jones <davej@redhat.com>
- 2.6.20rc1-git5

* Sun Dec 17 2006 David Woodhouse <dwmw2@redhat.com>
- Enable Efika platform support
- Temporarily provide __ucmpdi2 on ppc32 to work around GCC PR #25724

* Thu Dec 14 2006 Dave Jones <davej@redhat.com>
- 2.6.20rc1-git1
- libata: don't initialize sg in ata_exec_internal() if DMA_NONE

* Wed Dec 13 2006 Dave Jones <davej@redhat.com>
- Kill off -kdump for 686.

* Wed Dec 13 2006 David Woodhouse <dwmw2@redhat.com>
- Fix and re-enable ppc, ppc64, ppciseries builds
- Fix hdrcheck.sh invocation
- %%_target_cpu in BuildRoot to allow parallel builds

* Tue Dec 12 2006 Dave Jones <davej@redhat.com>
- 2.6.19-git19

* Tue Dec 12 2006 Dave Jones <davej@redhat.com>
- 2.6.19-git18

* Fri Dec  8 2006 Dave Jones <davej@redhat.com>
- 2.6.19-git12

* Thu Dec  7 2006 Dave Jones <davej@redhat.com>
- 2.6.19-git9

* Tue Dec  5 2006 David Woodhouse <dwmw2@redhat.com>
- Remove cell_defconfig update from Mambo patch
- Drop 64KiB CI page patch which is fixed differently upstream

* Tue Dec  5 2006 Dave Jones <davej@redhat.com>
- 2.6.19-git7

* Mon Dec  4 2006 Dave Jones <davej@redhat.com>
- 2.6.19-git5

* Wed Nov 29 2006 Dave Jones <davej@redhat.com>
- 2.6.19

* Mon Nov 27 2006 Dave Jones <davej@redhat.com>
- 2.6.19-rc6-git10

* Thu Nov 23 2006 David Woodhouse <dwmw2@redhat.com>
- Update Cell systemsim support (#217078)
- Fix utrace vs. Cell SPU (#217079)
- Enable Maple support

* Fri Nov 17 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc6
- Merge squashfs changes from FC6.
- Disable W1

* Fri Nov 10 2006 Dave Jones <davej@redhat.com>
- Separate -debug kernels.

* Thu Nov  9 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc5-git2
- Bump HZ to 1000

* Wed Nov  8 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc5-git1

* Tue Nov  7 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc5

* Mon Nov  6 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc4-git11

* Tue Oct 31 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc4

* Sun Oct 29 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc3

* Fri Oct 13 2006 Dave Jones <davej@redhat.com>
- 2.6.19rc2
